@@ -188,6 +188,7 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
     // We enforce metadata + spec's existence (always there)
     // => No default impl
     let rootident = Ident::new(&struct_name, Span::call_site());
+    let rootident_str = rootident.to_string();
 
     // if status set, also add that
     let StatusInformation {
@@ -196,9 +197,22 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
         impl_hasstatus,
     } = process_status(&rootident, &status, &visibility, &kube_core);
     let has_status = status.is_some();
+    let serialize_status = if has_status {
+        quote! {
+            if let Some(status) = &self.status {
+                obj.serialize_field("status", &status)?;
+            }
+        }
+    } else {
+        quote! {}
+    };
+    let has_status_value = if has_status {
+        quote! { self.status.is_some() }
+    } else {
+        quote! { false }
+    };
 
     let mut derive_paths: Vec<Path> = vec![
-        syn::parse_quote! { #serde::Serialize },
         syn::parse_quote! { #serde::Deserialize },
         syn::parse_quote! { Clone },
         syn::parse_quote! { Debug },
@@ -239,10 +253,6 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
         #[derive(#(#derive_paths),*)]
         #[serde(rename_all = "camelCase")]
         #visibility struct #rootident {
-            #schemars_skip
-            #visibility api_version: String,
-            #schemars_skip
-            #visibility kind: String,
             #schemars_skip
             #visibility metadata: #k8s_openapi::apimachinery::pkg::apis::meta::v1::ObjectMeta,
             #visibility spec: #ident,
@@ -251,8 +261,6 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
         impl #rootident {
             pub fn new(name: &str, spec: #ident) -> Self {
                 Self {
-                    api_version: <#rootident as #kube_core::Resource>::api_version(&()).to_string(),
-                    kind: <#rootident as #kube_core::Resource>::kind(&()).to_string(),
                     metadata: #k8s_openapi::apimachinery::pkg::apis::meta::v1::ObjectMeta {
                         name: Some(name.to_string()),
                         ..Default::default()
@@ -262,6 +270,18 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
                 }
             }
         }
+        impl #serde::Serialize for #rootident {
+            fn serialize<S: #serde::Serializer>(&self, ser: S) -> Result<S::Ok, S::Error> {
+                use #serde::ser::SerializeStruct;
+                let mut obj = ser.serialize_struct(#rootident_str, 4 + usize::from(#has_status_value))?;
+                obj.serialize_field("apiVersion", &<#rootident as #kube_core::Resource>::api_version(&()))?;
+                obj.serialize_field("kind", &<#rootident as #kube_core::Resource>::kind(&()))?;
+                obj.serialize_field("metadata", &self.metadata)?;
+                obj.serialize_field("spec", &self.spec)?;
+                #serialize_status
+                obj.end()
+            }
+        }
     };
 
     // 2. Implement Resource trait
@@ -310,8 +330,6 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
             impl Default for #rootident {
                 fn default() -> Self {
                     Self {
-                        api_version: <#rootident as #kube_core::Resource>::api_version(&()).to_string(),
-                        kind: <#rootident as #kube_core::Resource>::kind(&()).to_string(),
                         metadata: #k8s_openapi::apimachinery::pkg::apis::meta::v1::ObjectMeta::default(),
                         spec: Default::default(),
                         #status_default
@@ -358,10 +376,13 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
     let schemagen = if schema_mode.use_in_crd() {
         quote! {
             // Don't use definitions and don't include `$schema` because these are not allowed.
-            let gen = #schemars::gen::SchemaSettings::openapi3().with(|s| {
-                s.inline_subschemas = true;
-                s.meta_schema = None;
-            }).into_generator();
+            let gen = #schemars::gen::SchemaSettings::openapi3()
+                .with(|s| {
+                    s.inline_subschemas = true;
+                    s.meta_schema = None;
+                })
+                .with_visitor(#kube_core::schema::StructuralSchemaRewriter)
+                .into_generator();
             let schema = gen.into_root_schema_for::<Self>();
         }
     } else {
